@@ -8,12 +8,9 @@
 import cgi
 import sys
 import timeit
-
+from StringIO import StringIO
 from genshi.builder import tag
 from genshi.template import MarkupTemplate
-import neo_cgi
-import neo_cs
-import neo_util
 
 try:
     from elementtree import ElementTree as et
@@ -43,6 +40,11 @@ try:
 except ImportError:
     DjangoContext = DjangoTemplate = None
 
+try:
+    from myghty.interp import Interpreter as MyghtyInterpreter
+except ImportError:
+    MyghtyInterpreter = None
+
 table = [dict(a=1,b=2,c=3,d=4,e=5,f=6,g=7,h=8,i=9,j=10)
           for x in range(1000)]
 
@@ -72,6 +74,24 @@ if DjangoTemplate:
         context = DjangoContext({'table': table})
         django_tmpl.render(context)
 
+if MyghtyInterpreter:
+    interpreter = MyghtyInterpreter()
+    component = interpreter.make_component("""
+<table>
+% for row in ARGS['table']:
+   <tr>
+%    for col in row.values():
+     <td><% col %></td>
+%
+%
+   </tr>
+</table>
+""")
+    def test_myghty():
+        """Myghty Template"""
+        buf = StringIO()
+        interpreter.execute(component, request_args={'table':table}, out_buffer=buf)
+
 def test_genshi():
     """Genshi template"""
     stream = genshi_tmpl.generate(table=table)
@@ -114,7 +134,6 @@ if kid:
         kid_tmpl.table = table
         kid_tmpl.serialize(output='html')
 
-
     if cet:
         def test_kid_et():
             """Kid template + cElementTree"""
@@ -166,9 +185,10 @@ if neo_cgi:
 
 
 def run(which=None, number=10):
-    tests = ['test_builder', 'test_genshi', 'test_genshi_builder', 'test_kid',
-             'test_kid_et', 'test_et', 'test_cet', 'test_clearsilver',
-             'test_django']
+    tests = ['test_builder', 'test_genshi', 'test_genshi_builder',
+             'test_myghty', 'test_kid', 'test_kid_et', 'test_et', 'test_cet',
+             'test_clearsilver', 'test_django']
+
     if which:
         tests = filter(lambda n: n[5:] in which, tests)
 
