@@ -38,6 +38,7 @@ from genshi.core import Stream, Attrs, Namespace, QName
 from genshi.core import START, END, TEXT, COMMENT, PI
 
 __all__ = ['Path', 'PathSyntaxError']
+__docformat__ = 'restructuredtext en'
 
 
 class Axis(object):
@@ -75,7 +76,7 @@ class Path(object):
     def __init__(self, text, filename=None, lineno=-1):
         """Create the path object from a string.
         
-        @param text: the path expression
+        :param text: the path expression
         """
         self.source = text
         self.paths = PathParser(text, filename, lineno).parse()
@@ -105,10 +106,10 @@ class Path(object):
         >>> print Path('.//child/text()').select(xml)
         Text
         
-        @param stream: the stream to select from
-        @param namespaces: (optional) a mapping of namespace prefixes to URIs
-        @param variables: (optional) a mapping of variable names to values
-        @return: the substream matching the path, or an empty stream
+        :param stream: the stream to select from
+        :param namespaces: (optional) a mapping of namespace prefixes to URIs
+        :param variables: (optional) a mapping of variable names to values
+        :return: the substream matching the path, or an empty stream
         """
         if namespaces is None:
             namespaces = {}
@@ -117,19 +118,21 @@ class Path(object):
         stream = iter(stream)
         def _generate():
             test = self.test()
-            for kind, data, pos in stream:
-                result = test(kind, data, pos, namespaces, variables)
+            for event in stream:
+                result = test(event, namespaces, variables)
                 if result is True:
-                    yield kind, data, pos
-                    depth = 1
-                    while depth > 0:
-                        subkind, subdata, subpos = stream.next()
-                        if subkind is START:
-                            depth += 1
-                        elif subkind is END:
-                            depth -= 1
-                        yield subkind, subdata, subpos
-                        test(subkind, subdata, subpos, namespaces, variables)
+                    yield event
+                    if event[0] is START:
+                        depth = 1
+                        while depth > 0:
+                            subevent = stream.next()
+                            if subevent[0] is START:
+                                depth += 1
+                            elif subevent[0] is END:
+                                depth -= 1
+                            yield subevent
+                            test(subevent, namespaces, variables,
+                                 updateonly=True)
                 elif result:
                     yield result
         return Stream(_generate())
@@ -138,30 +141,33 @@ class Path(object):
         """Returns a function that can be used to track whether the path matches
         a specific stream event.
         
-        The function returned expects the positional arguments `kind`, `data`,
-        `pos` (basically an unpacked stream event), as well as `namespaces`
-        and `variables`. The latter two are a mapping of namespace prefixes to
-        URIs, and a mapping of variable names to values, respectively.
+        The function returned expects the positional arguments ``event``,
+        ``namespaces`` and ``variables``. The first is a stream event, while the
+        latter two are a mapping of namespace prefixes to URIs, and a mapping
+        of variable names to values, respectively. In addition, the function
+        accepts an ``updateonly`` keyword argument that default to ``False``. If
+        it is set to ``True``, the function only updates its internal state,
+        but does not perform any tests or return a result.
         
         If the path matches the event, the function returns the match (for
-        example, a `START` or `TEXT` event.) Otherwise, it returns `None`.
+        example, a `START` or `TEXT` event.) Otherwise, it returns ``None``.
         
         >>> from genshi.input import XML
         >>> xml = XML('<root><elem><child id="1"/></elem><child id="2"/></root>')
         >>> test = Path('child').test()
-        >>> for kind, data, pos in xml:
-        ...     if test(kind, data, pos, {}, {}):
-        ...         print kind, data
-        START (u'child', [(u'id', u'2')])
+        >>> for event in xml:
+        ...     if test(event, {}, {}):
+        ...         print event[0], repr(event[1])
+        START (QName(u'child'), Attrs([(QName(u'id'), u'2')]))
         """
         paths = [(p, len(p), [0], [], [0] * len(p)) for p in [
             (ignore_context and [_DOTSLASHSLASH] or []) + p for p in self.paths
         ]]
 
-        def _test(kind, data, pos, namespaces, variables):
+        def _test(event, namespaces, variables, updateonly=False):
+            kind, data, pos = event[:3]
             retval = None
             for steps, size, cursors, cutoff, counter in paths:
-
                 # Manage the stack that tells us "where we are" in the stream
                 if kind is END:
                     if cursors:
@@ -170,7 +176,7 @@ class Path(object):
                 elif kind is START:
                     cursors.append(cursors and cursors[-1] or 0)
 
-                if retval or not cursors:
+                if updateonly or retval or not cursors:
                     continue
                 cursor = cursors[-1]
                 depth = len(cursors)
@@ -232,15 +238,15 @@ class Path(object):
 
                         elif steps[cursor][0] is ATTRIBUTE:
                             # If the axis of the next location step is the
-                            # attribute axis, we need to move on to
-                            # processing that step without waiting for the
-                            # next markup event
+                            # attribute axis, we need to move on to processing
+                            # that step without waiting for the next markup
+                            # event
                             continue
 
                     # We're done with this step if it's the last step or the
                     # axis isn't "self"
-                    if last_step or not (axis is SELF or
-                                         axis is DESCENDANT_OR_SELF):
+                    if not matched or last_step or not (
+                            axis is SELF or axis is DESCENDANT_OR_SELF):
                         break
 
                 if (retval or not matched) and kind is START and \
@@ -584,7 +590,7 @@ class CommentNodeTest(object):
     """Node test that matches any comment events."""
     __slots__ = []
     def __call__(self, kind, data, pos, namespaces, variables):
-        return kind is COMMENT and (kind, data, pos)
+        return kind is COMMENT
     def __repr__(self):
         return 'comment()'
 
@@ -604,8 +610,7 @@ class ProcessingInstructionNodeTest(object):
     def __init__(self, target=None):
         self.target = target
     def __call__(self, kind, data, pos, namespaces, variables):
-        if kind is PI and (not self.target or data[0] == self.target):
-            return (kind, data, pos)
+        return kind is PI and (not self.target or data[0] == self.target)
     def __repr__(self):
         arg = ''
         if self.target:
@@ -616,7 +621,7 @@ class TextNodeTest(object):
     """Node test that matches any text event."""
     __slots__ = []
     def __call__(self, kind, data, pos, namespaces, variables):
-        return kind is TEXT and (kind, data, pos)
+        return kind is TEXT
     def __repr__(self):
         return 'text()'
 
