@@ -247,7 +247,8 @@ class LookupBase(object):
         return {
             '_lookup_name': cls.lookup_name,
             '_lookup_attr': cls.lookup_attr,
-            '_lookup_item': cls.lookup_item
+            '_lookup_item': cls.lookup_item,
+            'UndefinedError': UndefinedError
         }
     globals = classmethod(globals)
 
@@ -671,7 +672,8 @@ class TemplateASTTransformer(ASTTransformer):
         return node
 
     def visitAugAssign(self, node):
-        if isinstance(node.node, ast.Name):
+        if isinstance(node.node, ast.Name) and (not self.locals
+                or node.node.name not in flatten(self.locals)):
             name = node.node.name
             node.node = ast.Subscript(ast.Name('data'), 'OP_APPLY',
                                       [ast.Const(name)])
@@ -686,6 +688,8 @@ class TemplateASTTransformer(ASTTransformer):
             return ASTTransformer.visitAugAssign(self, node)
 
     def visitClass(self, node):
+        if self.locals:
+            self.locals[-1].add(node.name)
         self.locals.append(set())
         node = ASTTransformer.visitClass(self, node)
         self.locals.pop()
@@ -698,6 +702,8 @@ class TemplateASTTransformer(ASTTransformer):
         return node
 
     def visitFunction(self, node):
+        if self.locals:
+            self.locals[-1].add(node.name)
         self.locals.append(set(node.argnames))
         node = ASTTransformer.visitFunction(self, node)
         self.locals.pop()
@@ -724,12 +730,11 @@ class TemplateASTTransformer(ASTTransformer):
     def visitName(self, node):
         # If the name refers to a local inside a lambda, list comprehension, or
         # generator expression, leave it alone
-        for frame in self.locals:
-            if node.name in frame:
-                return node
-        # Otherwise, translate the name ref into a context lookup
-        func_args = [ast.Name('data'), ast.Const(node.name)]
-        return ast.CallFunc(ast.Name('_lookup_name'), func_args)
+        if node.name not in flatten(self.locals):
+            # Otherwise, translate the name ref into a context lookup
+            func_args = [ast.Name('data'), ast.Const(node.name)]
+            node = ast.CallFunc(ast.Name('_lookup_name'), func_args)
+        return node
 
 
 class ExpressionASTTransformer(TemplateASTTransformer):
